/********************************************************************/
/*    TORQUE (Types Of Relationships QUality Evaluation) toolkit    */
/*                                                                  */
/* Original file name: policygraph.h                                */
/*                                                                  */
/* This software by R.M.                                            */
/********************************************************************/

#ifndef POLGRAPH_H
#define POLGRAPH_H

#include "graph.h"
#include <stdio.h>
#include <string.h>

// *********************************************
// ************** BGP Policy Graph *************
// *********************************************

typedef unsigned short int ASnode;

// Policies are distinguished into import and export policies
enum policy_type{POL_IMPORT = 0, POL_EXPORT = 1};

/* This structure is only used to support automatic memory
   allocation/release for edge labels */
struct Policy{
   private:
      static void release_mem(char**);     /* A simple support method that releases memory
                                              (if it had been allocated before) */
      char* import_policy;                 // Label describing the import policy
      char* export_policy;                 // Label describing the export policy
   public:
      Policy();                            // Constructor (allocates memory)
      ~Policy();                           // Destructor  (releases allocated memory)
      Policy(const Policy&);               // Copy constructor
      Policy& operator=(const Policy&);    // Assignment operator
      bool oriented;                       /* This flag is true when the edge has been assigned
                                              a valid orientation; false otherwise */
      void setPolicy(policy_type, char*);  /* This method can be used to set the import/export policy of an
                                              AS. Considering an edge AS1-AS2, this label represents
                                              the import/export policy of AS1 */
      char* getPolicy(policy_type);        /* This method returns the current import/export policy of an AS,
                                              if any (returns an empty string if a policy has not been set yet).
                                              Given an edge AS1-AS2, the method returns the import
                                              policy of AS1. */
};

class PolicyGraph: public Graph<ASnode, Policy, ASnode> {
   private:
      char error[513];                          /* This variable stores a short
                                                   explanation for error conditions,
                                                   when they occur; 513 includes 512
                                                   characters plus terminating '\0' */
      int dirToVal(edge_dir);                   /* This funcion converts an edge direction
                                                   (IN, OUT, NONE) into a printable integer
                                                   value (-1, 1, 0) */
      edge_dir valToDir(int);                   /* This function converts an integer value
                                                   (-1, 1, 0) into an edge direction
                                                   (IN, OUT, NONE) */
      /* The following constant stores the 3-characters tag that identifies
         policy graphs when being written to file */
      static const char policyGraphMarker[4];

   public:
      PolicyGraph();                    // Default constructor
      ~PolicyGraph();                   // Destructor
      bool enhanced;                    /* If this flag is true, then
                                           this graph has been enhanced with
                                           "real" routing information */
      bool writeToFile(FILE*);          /* Dumps all the policy graph contents to an
                                           ASCII file; use `readFromFile' to
                                           get them back; the only parameter is
                                           a pointer to an open file strem;
                                           this method returns true if operation
                                           is successful; false otherwise */
      bool readFromFile(FILE*);         /* Builds a policy graph from the data that have
                                           previously been generated by `writeToFile';
                                           the only parameter is a pointer to an
                                           open file stream; returns true if
                                           operation is successful; false otherwise */
      char *errReason();                /* Returns a string that describes the
                                           reason of last file error; this method
                                           should be invoked after that either one
                                           of `writeToFile' or `readFromFile'
                                           returned false */
};

// ************** BGP Policy graph *************

#endif
